var _, moment, rdb, utils;

_ = require('lodash');

moment = require('moment');

utils = require('./utils');

rdb = require('rethinkdb');

module.exports = function(con, config) {
  var API, Groups, hasResult, isFinalized, isFree, lockSolutionForTutor, tutorExerciseStatsQuery;
  config.maxSolutionLocks = config.maxSolutionLocks || 10;
  isFree = function(doc, tutor) {
    if (tutor) {
      return doc.hasFields("results").not().and(doc.hasFields("lock").not().or(doc("lock").eq(tutor).or(doc("lock").eq(""))));
    } else {
      return doc.hasFields("results").not().and(doc.hasFields("lock").not().or(doc("lock").eq("")));
    }
  };
  hasResult = function(doc) {
    return doc.hasFields("results");
  };
  isFinalized = function(doc) {
    return doc.hasFields("results").and(doc("inProcess").not());
  };
  lockSolutionForTutor = function(tutor, id) {
    return rdb["do"](rdb.table("Solutions").get(id), function(doc) {
      return rdb.branch(isFree(doc, tutor), rdb.table("Solutions").get(id).update({
        lock: tutor,
        inProcess: true,
        lockTimeStamp: rdb.now()
      }), rdb.error("Solution (ID " + id + ") is already locked"));
    }).run(con);
  };
  tutorExerciseStatsQuery = function(name, exercise_id) {
    return rdb["do"](rdb.table("Tutors").sum("contingent"), rdb.table("Tutors").filter({
      name: name
    }).sum("contingent"), rdb.table("Solutions").filter({
      exercise: exercise_id
    }).count(), rdb.table("Solutions").filter({
      exercise: exercise_id,
      lock: name,
      inProcess: false
    }).count(), function(totalContingent, tutorContingent, solutionsCount, tutorSols) {
      return {
        is: tutorSols,
        should: solutionsCount.mul(tutorContingent).div(totalContingent)
      };
    });
  };
  Groups = (require('./groups'))(con, config);
  return API = {
    getStatus: function(name) {
      return (Promise.all([
        utils.toArray(rdb.table("Exercises").run(con)), rdb.table("Tutors").sum("contingent").run(con), rdb.table("Tutors").filter({
          name: name
        }).sum("contingent").run(con), rdb.table("Solutions").group("exercise").count().run(con), rdb.table("Solutions").group("exercise").filter(isFinalized).count().run(con), rdb.table("Solutions").group("exercise").filter(function(doc) {
          return doc.hasFields("lock").and(doc("lock").ne(""));
        }).count().run(con), rdb.table("Solutions").filter({
          lock: name,
          inProcess: false
        }).group("exercise").count().run(con)
      ])).then(function(values) {
        var exerciseMap, perc_contingent;
        perc_contingent = values[2] / values[1];
        exerciseMap = {};
        _.each(values[0], function(v) {
          return exerciseMap[v.id] = {
            exercise: v,
            is: 0,
            should: 0,
            solutions: 0,
            corrected: 0,
            locked: 0
          };
        });
        _.each(values[3], function(v) {
          exerciseMap[v.group].solutions = v.reduction;
          return exerciseMap[v.group].should = v.reduction * perc_contingent;
        });
        _.each(values[4], function(v) {
          return exerciseMap[v.group].corrected = v.reduction;
        });
        _.each(values[5], function(v) {
          return exerciseMap[v.group].locked = v.reduction;
        });
        _.each(values[6], function(v) {
          return exerciseMap[v.group].is = v.reduction;
        });
        return _.values(exerciseMap);
      });
    },
    getExerciseContingentForTutor: function(name, exercise_id) {
      return tutorExerciseStatsQuery(name, exercise_id).run(con);
    },
    getResultsForExercise: function(exercise_id) {
      return utils.toArray(rdb.table("Solutions").getAll(exercise_id, {
        index: "exercise"
      }).run(con));
    },
    setResultForExercise: function(tutor, id, result) {
      return rdb["do"](rdb.table("Solutions").get(id), function(doc) {
        return rdb.branch(doc.hasFields("lock").and(doc("lock").eq(tutor)), rdb.table("Solutions").get(id).update({
          results: result
        }), rdb.error("Only locked solutions can be updated"));
      }).run(con);
    },
    finishSolution: function(tutor, id) {
      return rdb["do"](rdb.table("Solutions").get(id), function(doc) {
        return rdb.branch(doc.hasFields("lock").and(doc("lock").eq(tutor).and(doc.hasFields("results"))), rdb.table("Solutions").get(id).update({
          inProcess: false
        }), rdb.error("Cannot finish solution, are you missing a result or are not authorized?"));
      }).run(con);
    },
    hasPdf: function(solutionId) {
      return rdb.table("Solutions").get(solutionId).getField("processed").run(con);
    },
    getUserSolutions: function(user) {
      return Groups.getGroupForUser(user).then(function(group) {
        return utils.toArray((rdb.table("Solutions").filter(function(s) {
          return s("group").eq(group.id);
        })).run(con));
      });
    },
    getUserExerciseSolution: function(user, exercise_id) {
      return Groups.getGroupForUser(user).then(function(group) {
        return (rdb.table("Solutions").filter({
          group: group.id,
          exercise: exercise_id
        }).nth(0)).run(con);
      });
    },
    getSolutionsForExercise: function(exercise_id) {
      return utils.toArray(rdb.table("Solutions").getAll(exercise_id, {
        index: "exercise"
      }).without("results").run(con));
    },
    getSolutionById: function(solution_id) {
      return rdb.table("Solutions").get(solution_id).run(con);
    },
    getSolutionTasks: function(solution_id) {
      return rdb.table("Solutions").get(solution_id)("tasks").run(con);
    },
    getSolutionsForGroup: function(group_id) {
      return utils.toArray(rdb.table("Solutions").getAll(group_id, {
        index: "group"
      }).run(con));
    },

    /*
    getSolutionsForUser: (name) ->
      utils.toArray rdb.table("Groups").getAll(name, {index: })
     */
    getPDFForID: function(id) {
      return utils.getFirstKey("pdf", rdb.table("Solutions").get(id).run(con));
    },
    getLockedSolutionsForExercise: function(exercise_id) {
      return utils.toArray(rdb.table("Solutions")).getAll(exercise_id, {
        index: "exercise"
      }).filter(function(s) {
        return s.hasFields('lock').and(s('lock').ne(""));
      }).without("results").run(con);
    },
    getFinishedSolutionsForTutor: function(tutor) {
      return utils.toArray(rdb.table("Solutions").getAll(tutor, {
        index: "lock"
      }).filter(function(doc) {
        return doc("inProcess").not();
      }).run(con));
    },
    getUnfinishedSolutionsForTutor: function(tutor) {
      return utils.toArray(rdb.table("Solutions").getAll(tutor, {
        index: "lock"
      }).filter(function(doc) {
        return doc("inProcess");
      }).run(con));
    },
    lockNextSolutionForTutor: function(tutor, exercise_id) {
      return (rdb.table("Solutions").getAll(exercise_id, {
        index: "exercise"
      }).filter(function(doc) {
        return isFree(doc);
      }).sample(1).run(con)).then(function(sol) {
        return rdb.table("Solutions").getAll(exercise_id, {
          index: "exercise"
        }).filter(rdb.row("lock").eq(tutor).and(rdb.row("inProcess").eq(true))).count().run(con).then(function(lockCount) {
          if (lockCount > config.maxSolutionLocks) {
            return Promise.reject("Too much locked solutions by tutor " + tutor);
          }
          if (!sol || sol.length !== 1) {
            return Promise.resolve();
          }
          return rdb.table("Solutions").get(sol[0].id).update({
            lock: tutor,
            inProcess: true,
            lockTimeStamp: rdb.now()
          }).run(con).then(function() {
            sol[0].inProcess = true;
            sol[0].lock = tutor;
            sol[0].lockTimeStamp = rdb.now();
            sol[0].lockCount = lockCount;
            return sol[0];
          });
        });
      });
    },
    getNumPending: function(exercise_id) {
      return rdb.table("Solutions").filter(function(doc) {
        return doc.hasFields("results");
      }).count().run(con);
    },
    lockSolutionForTutor: function(tutor, id) {
      return lockSolutionForTutor(tutor, id);
    },
    checkResults: function(id) {
      return API.getSolutionById(id).then(function(sol) {
        var isNumber, numbers;
        isNumber = function(n) {
          return typeof n === 'number';
        };
        if (!sol.results) {
          return Promise.reject('Solution contains no result object');
        }
        if (!sol.results.points || !Array.isArray(sol.results.points)) {
          return Promise.reject('Points in Solution must be an Array');
        }
        numbers = _.filter(sol.results.points, isNumber);
        if (numbers.length !== sol.results.points.length) {
          return Promise.reject('Not all points are stored as numbers');
        }
        return require('./exercises')(con, config).getById(sol.exercise).then(function(e) {
          return {
            exercise: e,
            solution: sol
          };
        });
      }).then(function(val) {
        if (val.exercise.tasks.length !== val.solution.results.points.length) {
          return Promise.reject('Task number does not match number of results. (Tasks: ' + val.exercise.tasks.length + ' , Results: ' + val.solution.results.points.length + ')');
        }
        return Promise.resolve();
      });
    }
  };
};
