var _, moment, rdb, utils;

_ = require('lodash');

rdb = require('rethinkdb');

moment = require('moment');

utils = require('./utils');

module.exports = function(con, config) {
  var API, Corrections, Group, createAndReplaceSolutionForUser, createGroupSolutionAndUpdateUser, findSolutionForExerciseInUserArray, getExerciseSolution, getGroupSolutionForExercise;
  Group = (require('./groups'))(con, config);
  Corrections = (require('./corrections'))(con, config);
  getGroupSolutionForExercise = function(group_id, exercise_id) {
    return rdb.table("Solutions").getAll(group_id, {
      index: "group"
    }).filter({
      "exercise": exercise_id
    }).nth(0)["default"](null);
  };
  findSolutionForExerciseInUserArray = function(user_id, exercise_id) {
    return rdb.branch(rdb.table("Users").get(user_id)("solutions").count().ne(0), rdb.table("Solutions").getAll(rdb.args(rdb.table("Users").get(user_id)("solutions"))).filter({
      "exercise": exercise_id
    }).nth(0)["default"](null), rdb.expr(null));
  };
  getExerciseSolution = function(user_id, exercise_id) {
    return (Group.getGroupForUser(user_id)).then(function(group) {
      return utils.firstOrEmpty(rdb.table("Solutions").getAll(group.id, {
        index: "group"
      }).filter(function(doc) {
        return doc("exercise").eq(exercise_id);
      }).run(con)).then(function(sol) {
        if (!sol) {
          return null;
        }
        if (sol.inProcess) {
          delete sol.lock;
          delete sol.results;
        }
        delete sol.inProcess;
        return sol;
      });
    });
  };
  createAndReplaceSolutionForUser = function(group_id, user_id, solution_id, exercise_id) {

    /*
    if group has solution for exercise
      remove old solution from user
      assign group solution to user
    else
      copy solution of user to group
     */
    return rdb.branch(rdb.table("Solutions").getAll(group_id, {
      index: "group"
    }).count().ne(0), rdb["do"](getGroupSolutionForExercise(group_id, exercise_id), function(newSolution) {
      return rdb["do"](rdb.table("Users").get(user_id).update({
        solutions: rdb.table("Users").get(user_id)("solutions").difference([solution_id])
      }, {
        nonAtomic: true
      }), function() {
        return rdb.table("Users").get(user_id).update(function() {
          return {
            solutions: rdb.table("Users").get(user_id)("solutions").append(newSolution("id"))
          };
        }, {
          nonAtomic: true
        });
      });
    }), rdb.table("Solutions").insert({
      group: group_id,
      exercise: exercise_id,
      tasks: rdb.table("Solutions").get(solution_id).getField("tasks"),
      lastStore: rdb.epochTime(0)
    }));
  };
  createGroupSolutionAndUpdateUser = function(group_id, exercise_id, groupSolution, user_id) {
    if (groupSolution != null) {
      return rdb.table("Users").get(user_id).update({
        solutions: rdb.row("solutions").append(groupSolution.id)
      }).run(con);
    } else {
      return rdb.table("Solutions").insert({
        group: group_id,
        exercise: exercise_id,
        tasks: [],
        lastStore: rdb.epochTime(0)
      }, {
        return_changes: true
      }).run(con).then(function(changes) {
        return rdb.table("Users").get(user_id).update({
          solutions: rdb.row("solutions").append(changes.generated_keys[0])
        }).run(con);
      });
    }
  };
  return API = {
    get: function() {
      return rdb.table('Exercises').without("internals").without({
        tasks: ['solution', 'solutionTests', 'internals']
      }).filter(function(ex) {
        return ex('activationDate').lt(new Date());
      }).coerceTo('array').run(con);
    },
    getById: function(id) {
      return rdb.table("Exercises").get(id).without('internals').without({
        tasks: ['solution', 'solutionTests', 'internals']
      }).run(con).then(function(e) {
        return new Promise(function(resolve, reject) {
          if (moment().isAfter(e.activationDate)) {
            return resolve(e);
          } else {
            return reject("Exercise with id " + id + " not found");
          }
        });
      });
    },
    getAllActive: function() {
      return utils.toArray(rdb.table('Exercises').filter(function(ex) {
        return ex('activationDate').lt(new Date()).and(ex('dueDate').gt(new Date()));
      }).without('internals').without({
        tasks: ['solution', 'solutionTests', 'internals']
      }).run(con));
    },
    isActive: function(id) {
      return utils.nonEmptyList(rdb.table("Exercises").getAll(id).filter(function(ex) {
        return ex('activationDate').lt(new Date()).and(ex('dueDate').gt(new Date()));
      }).run(con));
    },
    getExerciseSolution: function(user_id, exercise_id) {
      return getExerciseSolution(user_id, exercise_id);
    },
    createExerciseSolution: function(user_id, exercise_id) {
      return (API.isActive(exercise_id)).then(function(active) {
        if (!active) {
          return Promise.reject("Cannot change solution for an exercise that is not active (user " + user_id + ", exercise: " + exercise_id + ")");
        } else {
          return Group.getGroupForUserUnfiltered(user_id).then(function(group) {
            return findSolutionForExerciseInUserArray(user_id, exercise_id).run(con).then(function(solution) {
              return getGroupSolutionForExercise(group.id, exercise_id).run(con).then(function(groupSolution) {
                if (solution != null) {
                  return rdb.branch(rdb.table("Solutions").getAll(rdb.args(rdb.table("Users").get(user_id).getField("solutions"))).filter({
                    "exercise": exercise_id,
                    "group_id": group.id
                  }).count().ne(0), rdb.expr("Solution already exists"), createAndReplaceSolutionForUser(group.id, user_id, solution.id, exercise_id)).run(con);
                } else {
                  return createGroupSolutionAndUpdateUser(group.id, exercise_id, groupSolution, user_id);
                }
              });
            });
          });
        }
      });
    },
    getSolutions: function() {
      return rdb.table("Solutions").coerceTo('array').run(con);
    },
    getPDFForID: function(user_id, exercise_id) {
      return rdb.table("Users").get(user_id)('solutions').map(function(sid) {
        return rdb.table("Solutions").get(sid);
      }).filter(function(sol) {
        return sol('exercise').eq(exercise_id);
      })('pdf').nth(0).run(con);
    },
    getCorrectedPDFForID: function(user_id, exercise_id) {
      return rdb.table("Users").get(user_id)('solutions').map(function(sid) {
        return rdb.table("Solutions").get(sid);
      }).filter(function(sol) {
        return sol('exercise').eq(exercise_id);
      })('correctedPDF').nth(0).run(con);
    }
  };
};
