var _, moment, rdb, utils;

_ = require('lodash');

rdb = require('rethinkdb');

moment = require('moment');

utils = require('./utils');

module.exports = function(con, config) {
  var createGroup, desensetizeGroup, desensetizeGroupQuery, desensetizeGroups, getGroupForUser, idListToPseudonymList, leaveGroup, pseudonymListToIdList, removeFromGroup, removePendingUsers, updatePreviousGroup;
  desensetizeGroupQuery = function(query) {
    return query.merge(function(g) {
      return {
        users: idListToPseudonymList(g("users")),
        pendingUsers: idListToPseudonymList(g("pendingUsers"))
      };
    });
  };
  updatePreviousGroup = function(user_id, get_group_query) {
    return rdb.branch(rdb.table("Users").get(user_id).hasFields("previousGroup").not(), rdb.table("Users").get(user_id).update({
      "previousGroup": rdb.table("Groups").getAll(user_id, {
        index: "users"
      }).nth(0)("id").coerceTo('array')
    }, {
      nonAtomic: true
    }), rdb.table("Users").get(user_id).update({
      "previousGroup": rdb.row("previousGroup").append(rdb.table("Groups").getAll(user_id, {
        index: "users"
      }).nth(0)("id"))
    }, {
      nonAtomic: true
    }));
  };
  removeFromGroup = function(user_id) {
    return rdb.table("Groups").getAll(user_id, {
      index: "users"
    }).replace(function(doc) {
      return doc.merge({
        users: doc("users").setDifference([user_id])
      });
    });
  };
  getGroupForUser = function(user_id) {
    return desensetizeGroupQuery(rdb.table("Groups").getAll(user_id, {
      index: "users"
    }).nth(0));
  };
  leaveGroup = function(user_id) {
    return rdb["do"](updatePreviousGroup(user_id, getGroupForUser), removeFromGroup(user_id), function(updateStats, replaceStats) {
      return {
        updateStats: updateStats,
        replaceStats: replaceStats
      };
    });
  };
  pseudonymListToIdList = function(plist) {
    return rdb.expr(plist).map(function(p) {
      return rdb.table("Users").getAll(p, {
        index: "pseudonym"
      }).nth(0)("id");
    });
  };
  idListToPseudonymList = function(idlist) {
    return idlist.map(function(id) {
      return rdb.table("Users").get(id)("pseudonym");
    });
  };
  desensetizeGroup = function(query) {
    return desensetizeGroupQuery(query).run(con);
  };
  desensetizeGroups = function(query) {
    return desensetizeGroupQuery(query).run(con);
  };
  createGroup = function(user_id, group_users) {
    return rdb["do"](pseudonymListToIdList(group_users), leaveGroup(user_id), function(grp_users, grp) {
      return rdb.table("Groups").insert({
        users: [user_id],
        pendingUsers: grp_users.setDifference([user_id])
      });
    }).run(con).then(function(res) {
      return desensetizeGroup(rdb.table("Groups").getAll(user_id, {
        index: "users"
      }).nth(0));
    });
  };
  removePendingUsers = function(group_id) {
    return rdb.table("Groups").get(group_id).update({
      pendingUsers: []
    });
  };
  return {
    create: function(user_id, group_users) {
      return createGroup(user_id, group_users);
    },
    hasGroup: function(user_id) {
      return rdb.table("Groups").getAll(user_id, {
        index: "users"
      }).count().run(con).then(function(cnt) {
        return cnt !== 0;
      });
    },
    getGroupForUser: function(user_id) {
      return getGroupForUser(user_id).run(con);
    },
    getGroupForUserUnfiltered: function(user_id) {
      return rdb.table("Groups").getAll(user_id, {
        index: "users"
      }).nth(0).run(con);
    },
    getGroupForUserQuery: function(user_id) {
      return getGroupForUser(user_id);
    },
    pending: function(user_id) {
      return utils.toArray(desensetizeGroups(rdb.table("Groups").getAll(user_id, {
        index: "pendingUsers"
      })));
    },
    rejectInvitation: function(user_id, group_id) {

      /*
      rdb.table("Groups").get(group_id).update({
        pendingUsers: rdb.table("Groups").get(group_id)("pendingUsers").setDifference([user_id])
      }, nonAtomic: true ).run(con)
       */
      return (rdb.table("Groups").get(group_id).replace(function(doc) {
        return doc.merge({
          pendingUsers: doc("pendingUsers").setDifference([user_id])
        });
      })).run(con);
    },
    joinGroup: function(user_id, group_id) {
      return (rdb.branch(rdb.table("Groups").get(group_id)("pendingUsers").contains(user_id), rdb["do"](leaveGroup(user_id), rdb.table("Groups").get(group_id).replace(function(doc) {
        return doc.merge({
          pendingUsers: doc("pendingUsers").setDifference([user_id]),
          users: doc("users").setUnion([user_id])
        });
      }), getGroupForUser(user_id)["default"]({
        failed: user_id
      }), function(res1, res2, res3) {
        return res3;
      }), rdb.error("User cannot join a group without invitation"))).run(con);
    },
    getGroups: function() {
      return rdb.table("Groups").coerceTo('array').run(con);
    },
    getGroup: function(group_id) {
      return rdb.table("Groups").get(group_id).run(con);
    },
    leaveGroupOnly: function(user_id) {
      return leaveGroup(user_id).run(con);
    },
    leaveGroup: function(user_id) {
      return (require('./users'))(con, config).getPseudonym(user_id).then(function(pseudo) {
        return getGroupForUser(user_id).run(con).then(function(group) {
          if (group.users.length === 1) {
            removePendingUsers(group.id).run(con).then(function() {});
          }
          return createGroup(user_id, [pseudo]);
        });
      });
    }
  };
};
