var ArrayQuery, ObjectQuery, connect;

connect = function(rdb, host, port) {
  return rdb.connect({
    host: host,
    port: port
  });
};

ArrayQuery = function(query, con, cb) {
  con.use('TutorDB');
  return query.run(con, function(err, cursor) {
    if (err) {
      return cb(err, null);
    } else {
      return cursor.toArray(function(err, results) {
        return cb(err, results);
      });
    }
  });
};

ObjectQuery = function(query, con, cb) {
  con.use('TutorDB');
  return query.run(con, function(err, singleResult) {
    if (err) {
      return cb(err, null);
    } else {
      return cb(err, singleResult);
    }
  });
};

module.exports = function(config) {
  return {
    DB: {
      Exercises: {
        name: 'Exercises',
        getAll: function(cb) {
          var conProm, query, rdb;
          rdb = require('rethinkdb');
          conProm = connect(rdb, config.database.host, config.database.port);
          query = rdb.table('Exercises');
          return conProm.then(function(con) {
            return ArrayQuery(query, con, function(err, data) {
              con.close();
              return cb(err, data);
            });
          });
        },
        getById: function(id, cb) {
          var conProm, query, rdb;
          rdb = require('rethinkdb');
          conProm = connect(rdb, config.database.host, config.database.port);
          query = rdb.table('Exercises').get(id);
          return conProm.then(function(con) {
            return ObjectQuery(query, con, function(err, data) {
              con.close();
              return cb(err, data);
            });
          });
        },
        getAllActive: function(cb) {
          var conProm, query, rdb;
          rdb = require('rethinkdb');
          conProm = connect(rdb, config.database.host, config.database.port);
          query = rdb.table('Exercises').filter(rdb.row('activationDate').le(rdb.now()));
          return conProm.then(function(con) {
            return ArrayQuery(query, con, function(err, data) {
              con.close();
              return cb(err, data);
            });
          });
        },
        getDetailedExercise: function(id, cb) {
          var conProm, rdb;
          rdb = require('rethinkdb');
          conProm = connect(rdb, config.database.host, config.database.port);
          return conProm.then(function(con) {
            var query;
            query = rdb.table('Exercises').get(id);
            return ObjectQuery(query, con, function(err, exercise) {
              var taskTable;
              if (err || (exercise == null)) {
                con.close();
                return cb(err, exercise);
              } else {
                taskTable = rdb.table('Tasks');
                query = taskTable.getAll.apply(taskTable, exercise.tasks).coerceTo('array');
                return ObjectQuery(query, con, function(err, tasks) {
                  con.close();
                  exercise.tasks = tasks;
                  debugger;
                  return cb(err, exercise);
                });
              }
            });
          });
        }
      }
    }
  };
};
