var _, moment, rdb, rndString, utils;

_ = require('lodash');

rdb = require('rethinkdb');

utils = require('./utils');

rndString = require('randomstring');

moment = require('moment');

module.exports = function(con, config) {
  var API, Groups, clearPendingPseudonyms, generatePseudonymList;
  Groups = (require('./groups'))(con);
  config.lockTime = config.lockTime || 15;
  clearPendingPseudonyms = function() {
    return rdb.table('PseudonymList').hasFields("locked").filter(rdb.now().sub(rdb.row("locked")).gt(config.lockTime * 60))["delete"]();
  };
  generatePseudonymList = function(p) {
    return [p].concat(_.map(_.range(42), function() {
      return p + " " + _.random(1000, 9999);
    }));
  };
  return API = {
    exists: function(id) {
      return utils.nonEmpty(rdb.table('Users').get(id).run(con));
    },
    get: function(id) {
      return rdb.table('Users').get(id).run(con);
    },
    getPseudonym: function(id) {
      clearPendingPseudonyms();
      return utils.getFirstKey("pseudonym", rdb.table('Users').get(id).pluck('pseudonym').run(con));
    },
    setPseudonym: function(id, newPseudonym) {
      clearPendingPseudonyms();
      return rdb.branch(rdb.table('PseudonymList').getAll(newPseudonym).filter(function(doc) {
        return doc("user").ne(id);
      }).count().eq(0), rdb.table('Users').get(id).update({
        pseudonym: newPseudonym
      }), rdb.error("Pseudonym " + newPseudonym + " already taken")).run(con);
    },
    getTotalPointsQuery: function(user) {
      return rdb.branch(user("solutions").count().ne(0), rdb.table("Solutions").getAll(rdb.args(user("solutions"))).filter(function(solution) {
        return solution.hasFields("results").and(solution.hasFields({
          results: 'points'
        })).and(solution("inProcess").eq(false));
      }).map(function(solution) {
        return solution("results")("points").sum()["default"](0);
      }).sum()["default"](0), rdb.expr(0));
    },
    getTotalPoints: function(user_id) {
      return API.getTotalPointsQuery(rdb.table("Users").get(user_id)).run(con);
    },
    create: function(user) {
      if (!user.id || !user.name || !user.pseudonym || !user.matrikel) {
        return Promise.reject("User information incomplete (" + JSON.stringify(user) + ")");
      }
      user.solutions = [];
      return rdb.table('Users').insert(user, {
        conflict: "update"
      }).run(con).then(function() {
        return API.lockRandomPseudonymFromList(user.id, generatePseudonymList(user.pseudonym)).then(function(p) {
          return API.setPseudonym(user.id, p);
        }).then(function() {
          return Groups.hasGroup(user.id);
        }).then(function(hasGroup) {
          if (!hasGroup) {
            return (Groups.create(user.id, [])).then(function(group) {
              return Promise.resolve();
            });
          }
          return Promise.resolve();
        });
      });
    },
    lockRandomPseudonymFromList: function(id, plist) {
      clearPendingPseudonyms();
      return utils.first(rdb.expr(plist).filter(function(e) {
        return rdb.table("PseudonymList").getAll(e, {
          index: "pseudonym"
        }).count().eq(0);
      }).sample(1).run(con)).then(function(p) {
        return rdb.table("PseudonymList").insert({
          pseudonym: p,
          user: id,
          locked: rdb.ISO8601(moment().toJSON())
        }).run(con).then(function() {
          return p;
        });
      });
    },
    getInternalPseudonymList: function() {
      clearPendingPseudonyms();
      return utils.toArray(rdb.table('PseudonymList').run(con));
    },
    getPseudonymList: function() {
      clearPendingPseudonyms();
      return utils.toArray(rdb.table('Users')('pseudonym').run(con));
    },
    getTutor: function(name) {
      return (rdb.table('Tutors').get(name).run(con)).then(function(tutor) {
        if (!tutor) {
          reject("Tutor " + name + " does not exist");
        } else {
          delete tutor.pw;
          return tutor;
        }
      });
    },
    authTutor: function(name, pw_compare) {
      return (rdb.table('Tutors').get(name).run(con)).then(function(tutor) {
        if (tutor) {
          return pw_compare(tutor.password);
        }
        return false;
      });
    },
    clearPendingPseudonyms: function() {
      return clearPendingPseudonyms().run(con);
    }
  };
};
