var _, rdb, rdbSetup, tables;

_ = require('lodash');

rdb = require('rethinkdb');

rdbSetup = require('rethinkdb-setup');

tables = {
  Exercises: "id",
  Solutions: ["id", "exercise", "group", "lock", "lastStore"],
  Users: [
    "id", "pseudonym", "previousGroup", {
      name: "solutions",
      options: {
        multi: true
      }
    }
  ],
  Groups: [
    "id", {
      name: "users",
      options: {
        multi: true
      }
    }, {
      name: "pendingUsers",
      options: {
        multi: true
      }
    }
  ],
  Tutors: "name",
  PseudonymList: ["pseudonym", "user"],
  session: "id",
  ShareJS: "id",
  Jobs: ["id", "name", "data", "type"]
};

module.exports = {
  toArray: function(promise) {
    return promise.then(function(cursor) {
      return cursor.toArray();
    });
  },
  first: function(promise) {
    return promise.then(function(cursor) {
      return cursor.next();
    });
  },
  firstOrEmpty: function(promise) {
    return promise.then(function(cursor) {
      return cursor.toArray().then(function(arr) {
        if (arr.length === 0) {
          return null;
        } else {
          return arr[0];
        }
      });
    });
  },
  firstAndCheck: function(promise) {
    return promise.then(function(cursor) {
      return cursor.toArray().then(function(arr) {
        return new Promise(function(resolve, reject) {
          if (arr.length > 1) {
            return reject("Expected exactly one result, but got more");
          } else {
            return resolve(arr[0]);
          }
        });
      });
    });
  },
  failIfNoUpdate: function(promise, err) {
    return promise.then(function(cursor) {
      return new Promise(function(resolve, reject) {
        if (cursor.replaced === 0) {
          return reject(err);
        } else {
          return resolve();
        }
      });
    });
  },
  checkError: function(promise) {
    return promise.then(function(cursor) {
      return new Promise(function(resolve, reject) {
        if (cursor.errors > 0) {
          reject(cursor.first_error);
        }
        return resolve();
      });
    });
  },
  nonEmpty: function(promise) {
    return promise.then(function(cursor) {
      return new Promise(function(resolve) {
        return resolve(cursor !== null);
      });
    });
  },
  nonEmptyList: function(promise) {
    return promise.then(function(cursor) {
      return cursor.toArray().then(function(arr) {
        return arr.length !== 0;
      });
    });
  },
  getFirstKey: function(key, promise) {
    return promise.then(function(cursor) {
      return new Promise(function(resolve) {
        return resolve(cursor[key]);
      });
    });
  },
  getAllKeys: function(key, promise) {
    return promise.then(function(cursor) {
      return cursor.toArray().then(function(arr) {
        return _.map(arr, function(v) {
          return v[key];
        });
      });
    });
  },
  init: function(con, config) {
    config = {
      db: config.db,
      tables: tables
    };
    return new Promise(function(resolve, reject) {
      return rdbSetup.setup(con, config, function(err) {
        if (err) {
          return reject(err);
        } else {
          return resolve();
        }
      });
    });
  },
  empty: function(con, config) {
    config = {
      db: config.db,
      tables: tables
    };
    return new Promise(function(resolve, reject) {
      return rdbSetup.empty(con, config, function(err) {
        if (err) {
          return reject(err);
        } else {
          return resolve();
        }
      });
    });
  },
  load: function(con, data) {
    return new Promise(function(resolve, reject) {
      return rdbSetup.load(con, data, function(err) {
        if (err) {
          return reject(err);
        } else {
          return resolve();
        }
      });
    });
  }
};
